@@ -2,12 +2,6 @@ cflags="$c_warnings $extra_flags"
 cxxflags="$cxx_warnings $base_cxxflags $extra_flags"
 extra_configs="$extra_configs $local_infile_configs"
 configure="./configure $base_configs $extra_configs"
-for arg
-do
-  # Escape special characters so they don't confuse eval
-  configure="$configure "`echo "$arg" | \
-  		sed -e 's,\([^a-zA-Z0-9_.=-]\),\\\\\1,g'`
-done
 
 commands="\
 $make -k clean || true 
@@ -23,12 +17,16 @@ autoconf   || (echo \"Can't execute autoconf\"    && exit 1)
 if [ -d gemini ]
 then
    (cd gemini && aclocal && autoheader && aclocal && automake && autoconf)
-fi
+fi"
 
+if [ -z "$just_clean" ]
+then
+commands="$commands
 CFLAGS=\"$cflags\" CXX=\"$CXX\" CXXFLAGS=\"$cxxflags\" CXXLDFLAGS=\"$CXXLDFLAGS\" \
 $configure"
+fi
 
-if [ -z "$just_configure" ]
+if [ -z "$just_configure" -a -z "$just_clean" ]
 then
   commands="$commands
 
@@ -45,7 +43,7 @@ strip sql/mysqld"
   fi
 fi
 
-if test -z "$nonono"
+if test -z "$just_print"
 then
   eval "set -x; $commands"
 else
