@@ -31,27 +31,33 @@
 extern "C" {
 #endif /* __cplusplus */
 
-enum enum_vio_type { VIO_CLOSED, VIO_TYPE_TCPIP, VIO_TYPE_SOCKET,
-		     VIO_TYPE_NAMEDPIPE, VIO_TYPE_SSL};
-
-#ifndef __WIN__
-#define HANDLE void *
-#endif
+enum enum_vio_type
+{
+  VIO_CLOSED, VIO_TYPE_TCPIP, VIO_TYPE_SOCKET, VIO_TYPE_NAMEDPIPE,
+  VIO_TYPE_SSL, VIO_TYPE_SHARED_MEMORY
+};
 
 Vio*	vio_new(my_socket sd, enum enum_vio_type type, my_bool localhost);
 #ifdef __WIN__
-Vio*	vio_new_win32pipe(HANDLE hPipe);
-#endif
+Vio* vio_new_win32pipe(HANDLE hPipe);
+Vio* vio_new_win32shared_memory(NET *net,HANDLE handle_file_map,
+                                HANDLE handle_map,
+                                HANDLE event_server_wrote,
+                                HANDLE event_server_read,
+                                HANDLE event_client_wrote,
+                                HANDLE event_client_read,
+                                HANDLE event_conn_closed);
+int vio_read_pipe(Vio *vio, gptr buf, int size);
+int vio_write_pipe(Vio *vio, const gptr buf, int size);
+int vio_close_pipe(Vio * vio);
+#else
+#define HANDLE void *
+#endif /* __WIN__ */
+
 void	vio_delete(Vio* vio);
 int	vio_close(Vio* vio);
-
-#ifdef EMBEDDED_LIBRARY
-void vio_reset(Vio *vio);
-#else
-void vio_reset(Vio* vio, enum enum_vio_type type,
-	       my_socket sd, HANDLE hPipe, my_bool localhost);
-#endif
-
+void    vio_reset(Vio* vio, enum enum_vio_type type,
+                  my_socket sd, HANDLE hPipe, my_bool localhost);
 int	vio_read(Vio *vio, gptr	buf, int size);
 int	vio_write(Vio *vio, const gptr buf, int size);
 int	vio_blocking(Vio *vio, my_bool onoff, my_bool *old_mode);
@@ -62,22 +68,20 @@ int	vio_fastsend(Vio *vio);
 int	vio_keepalive(Vio *vio, my_bool	onoff);
 /* Whenever we should retry the last read/write operation. */
 my_bool	vio_should_retry(Vio *vio);
-/* Check that operation was timed out */
-my_bool	vio_was_interrupted(Vio *vio);
 /* Short text description of the socket for those, who are curious.. */
 const char* vio_description(Vio *vio);
 /* Return the type of the connection */
 enum enum_vio_type vio_type(Vio* vio);
 /* Return last error number */
-int vio_errno(Vio*vio);
+int	vio_errno(Vio*vio);
 /* Get socket number */
 my_socket vio_fd(Vio*vio);
 /* Remote peer's address and name in text form */
-my_bool vio_peer_addr(Vio* vio, char *buf, uint16 *port);
+my_bool	vio_peer_addr(Vio* vio, char *buf, uint16 *port);
 /* Remotes in_addr */
-void vio_in_addr(Vio *vio, struct in_addr *in);
-my_bool vio_poll_read(Vio *vio,uint timeout);
-void vio_timeout(Vio *vio,uint timeout);
+void	vio_in_addr(Vio *vio, struct in_addr *in);
+my_bool	vio_poll_read(Vio *vio,uint timeout);
+void	vio_timeout(Vio *vio,uint which, uint timeout);
 
 #ifdef HAVE_OPENSSL
 #include <openssl/opensslv.h>
@@ -119,6 +123,12 @@ struct st_VioSSLAcceptorFd
 		      const char *ca_file,const char *ca_path,
 		      const char *cipher);
 Vio *new_VioSSL(struct st_VioSSLAcceptorFd *fd, Vio *sd, int state);
+#endif /* HAVE_OPENSSL */
+
+#ifdef HAVE_SMEM
+int vio_read_shared_memory(Vio *vio, gptr buf, int size);
+int vio_write_shared_memory(Vio *vio, const gptr buf, int size);
+int vio_close_shared_memory(Vio * vio);
 #endif
 
 #ifdef	__cplusplus
@@ -128,19 +138,18 @@ Vio *new_VioSSL(struct st_VioSSLAcceptorFd *fd, Vio *sd, int state);
 #if defined(HAVE_VIO) && !defined(DONT_MAP_VIO)
 #define vio_delete(vio) 			(vio)->viodelete(vio)
 #define vio_errno(vio)	 			(vio)->vioerrno(vio)
-#define vio_read(vio, buf, size) 		(vio)->read(vio,buf,size)
-#define vio_write(vio, buf, size) 		(vio)->write(vio, buf, size)
+#define vio_read(vio, buf, size)                ((vio)->read)(vio,buf,size)
+#define vio_write(vio, buf, size)               ((vio)->write)(vio, buf, size)
 #define vio_blocking(vio, set_blocking_mode, old_mode)\
  	(vio)->vioblocking(vio, set_blocking_mode, old_mode)
 #define vio_is_blocking(vio) 			(vio)->is_blocking(vio)
 #define vio_fastsend(vio)			(vio)->fastsend(vio)
 #define vio_keepalive(vio, set_keep_alive)	(vio)->viokeepalive(vio, set_keep_alive)
 #define vio_should_retry(vio) 			(vio)->should_retry(vio)
-#define vio_was_interrupted(vio) 		(vio)->was_interrupted(vio)
 #define vio_close(vio)				((vio)->vioclose)(vio)
 #define vio_peer_addr(vio, buf, prt)		(vio)->peer_addr(vio, buf, prt)
 #define vio_in_addr(vio, in)			(vio)->in_addr(vio, in)
-#define vio_timeout(vio, seconds)			(vio)->timeout(vio, seconds)
+#define vio_timeout(vio, which, seconds)	(vio)->timeout(vio, which, seconds)
 #endif /* defined(HAVE_VIO) && !defined(DONT_MAP_VIO) */
 
 /* This enumerator is used in parser - should be always visible */
@@ -153,7 +162,8 @@ enum SSL_type
   SSL_TYPE_SPECIFIED
 };
 
-#ifndef EMBEDDED_LIBRARY
+
+/* HFTODO - hide this if we don't want client in embedded server */
 /* This structure is for every connection on both sides */
 struct st_vio
 {
@@ -170,7 +180,7 @@ struct st_vio
   void    (*viodelete)(Vio*);
   int     (*vioerrno)(Vio*);
   int     (*read)(Vio*, gptr, int);
-  int     (*write)(Vio*, gptr, int);
+  int     (*write)(Vio*, const gptr, int);
   int     (*vioblocking)(Vio*, my_bool, my_bool *);
   my_bool (*is_blocking)(Vio*);
   int     (*viokeepalive)(Vio*, my_bool);
@@ -178,11 +188,21 @@ struct st_vio
   my_bool (*peer_addr)(Vio*, char *, uint16*);
   void    (*in_addr)(Vio*, struct in_addr*);
   my_bool (*should_retry)(Vio*);
-  my_bool (*was_interrupted)(Vio*);
   int     (*vioclose)(Vio*);
-  void	  (*timeout)(Vio*, unsigned int timeout);
+  void	  (*timeout)(Vio*, unsigned int which, unsigned int timeout);
   void	  *ssl_arg;
+#ifdef HAVE_SMEM
+  HANDLE  handle_file_map;
+  char    *handle_map;
+  HANDLE  event_server_wrote;
+  HANDLE  event_server_read;
+  HANDLE  event_client_wrote;
+  HANDLE  event_client_read;
+  HANDLE  event_conn_closed;
+  long    shared_memory_remain;
+  char    *shared_memory_pos;
+  NET     *net;
+#endif /* HAVE_SMEM */
 #endif /* HAVE_VIO */
 };
-#endif /* EMBEDDED_LIBRARY */
 #endif /* vio_violite_h_ */
