@@ -147,12 +147,13 @@ row_build_row_ref_in_tuple(
 	dtuple_t*	ref,	/* in/out: row reference built; see the
 				NOTE below! */
 	dict_index_t*	index,	/* in: index */
-	rec_t*		rec);	/* in: record in the index;
+	rec_t*		rec,	/* in: record in the index;
 				NOTE: the data fields in ref will point
 				directly into this record, therefore,
 				the buffer page of this record must be
 				at least s-latched and the latch held
 				as long as the row reference is used! */
+	trx_t*		trx);	/* in: transaction */
 /***********************************************************************
 From a row build a row reference with which we can search the clustered
 index record. */
