@@ -21,10 +21,13 @@
 #include "fulltext.h"
 #include <m_ctype.h>
 #include <my_tree.h>
+#include <queues.h>
 
-#define true_word_char(X)       (isalnum(X) || (X)=='_')
-#define misc_word_char(X)       ((X)=='\'')
-#define word_char(X)            (true_word_char(X) || misc_word_char(X))
+#define true_word_char(s,X)	(my_isalnum(s,X) || (X)=='_')
+#define misc_word_char(X)	((X)=='\'')
+#define word_char(s,X)		(true_word_char(s,X) || misc_word_char(X))
+
+#define FT_MAX_WORD_LEN_FOR_SORT 31
 
 #define COMPILE_STOPWORDS_IN
 
@@ -61,16 +64,6 @@
 #define NORM_SUM				   (docstat.nsum)
 #define NORM_COS			    (sqrt(docstat.nsum2))
 
-#ifdef EVAL_RUN
-/*
-extern ulong collstat;
-#define PIVOT_STAT  (docstat.uniq)
-#define PIVOT_SLOPE (0.69)
-#define PIVOT_PIVOT ((double)collstat/(info->state->records+1))
-#define NORM_PIVOT  ((1-PIVOT_SLOPE)*PIVOT_PIVOT+PIVOT_SLOPE*docstat.uniq)
-*/
-#endif /* EVAL_RUN */
-
 #define PIVOT_VAL (0.0115)
 #define NORM_PIVOT  (1+PIVOT_VAL*docstat.uniq)
 /*---------------------------------------------------------------*/
@@ -103,9 +96,6 @@ typedef struct st_ft_word {
   byte * pos;
   uint	 len;
   double weight;
-#ifdef EVAL_RUN
-  byte	 cnt;
-#endif /* EVAL_RUN */
 } FT_WORD;
 
 typedef struct st_ftb_param {
@@ -121,12 +111,12 @@ int is_stopword(char *word, uint len);
 
 uint _ft_make_key(MI_INFO *, uint , byte *, FT_WORD *, my_off_t);
 
-byte ft_get_word(byte **, byte *, FT_WORD *, FTB_PARAM *);
-byte ft_simple_get_word(byte **, byte *, FT_WORD *);
+byte ft_get_word(CHARSET_INFO *, byte **, byte *, FT_WORD *, FTB_PARAM *);
+byte ft_simple_get_word(CHARSET_INFO *, byte **, byte *, FT_WORD *);
 
 typedef struct _st_ft_seg_iterator {
   uint        num, len;
-  MI_KEYSEG  *seg;
+  HA_KEYSEG  *seg;
   const byte *rec, *pos;
 } FT_SEG_ITERATOR;
 
@@ -135,13 +125,15 @@ void _mi_ft_segiterator_dummy_init(const byte *, uint, FT_SEG_ITERATOR *);
 uint _mi_ft_segiterator(FT_SEG_ITERATOR *);
 
 void ft_parse_init(TREE *, CHARSET_INFO *);
-int ft_parse(TREE *, byte *, int);
+int ft_parse(TREE *, byte *, int, my_bool);
 FT_WORD * ft_linearize(TREE *);
-FT_WORD * _mi_ft_parserecord(MI_INFO *, uint, byte *, const byte *);
-uint _mi_ft_parse(TREE *parsed, MI_INFO *info, uint keynr, const byte *record);
+FT_WORD * _mi_ft_parserecord(MI_INFO *, uint, const byte *);
+uint _mi_ft_parse(TREE *, MI_INFO *, uint, const byte *, my_bool);
+
+FT_INFO *ft_init_nlq_search(MI_INFO *, uint, byte *, uint, uint, byte *);
+FT_INFO *ft_init_boolean_search(MI_INFO *, uint, byte *, uint, CHARSET_INFO *);
 
 extern const struct _ft_vft _ft_vft_nlq;
-FT_INFO *ft_init_nlq_search(MI_INFO *, uint, byte *, uint, my_bool);
 int ft_nlq_read_next(FT_INFO *, char *);
 float ft_nlq_find_relevance(FT_INFO *, byte *, uint);
 void ft_nlq_close_search(FT_INFO *);
@@ -150,10 +142,10 @@ my_off_t ft_nlq_get_docid(FT_INFO *);
 void ft_nlq_reinit_search(FT_INFO *);
 
 extern const struct _ft_vft _ft_vft_boolean;
-FT_INFO *ft_init_boolean_search(MI_INFO *, uint, byte *, uint, my_bool);
 int ft_boolean_read_next(FT_INFO *, char *);
 float ft_boolean_find_relevance(FT_INFO *, byte *, uint);
 void ft_boolean_close_search(FT_INFO *);
 float ft_boolean_get_relevance(FT_INFO *);
 my_off_t ft_boolean_get_docid(FT_INFO *);
 void ft_boolean_reinit_search(FT_INFO *);
+
