@@ -17,18 +17,20 @@
 /* Read and write key blocks */
 
 #include "myisamdef.h"
-#ifdef	__WIN__
-#include <errno.h>
-#endif
 
 	/* Fetch a key-page in memory */
 
 uchar *_mi_fetch_keypage(register MI_INFO *info, MI_KEYDEF *keyinfo,
-			 my_off_t page, uchar *buff, int return_buffer)
+			 my_off_t page, int level, 
+                         uchar *buff, int return_buffer)
 {
   uchar *tmp;
   uint page_size;
-  tmp=(uchar*) key_cache_read(info->s->kfile,page,(byte*) buff,
+  DBUG_ENTER("_mi_fetch_keypage");
+  DBUG_PRINT("enter",("page: %ld",page));
+
+  tmp=(uchar*) key_cache_read(info->s->key_cache,
+                             info->s->kfile, page, level, (byte*) buff,
 			     (uint) keyinfo->block_length,
 			     (uint) keyinfo->block_length,
 			     return_buffer);
@@ -39,7 +41,7 @@ uchar *_mi_fetch_keypage(register MI_INFO *info, MI_KEYDEF *keyinfo,
     DBUG_PRINT("error",("Got errno: %d from key_cache_read",my_errno));
     info->last_keypage=HA_OFFSET_ERROR;
     my_errno=HA_ERR_CRASHED;
-    return 0;
+    DBUG_RETURN(0);
   }
   info->last_keypage=page;
   page_size=mi_getint(tmp);
@@ -47,20 +49,23 @@ uchar *_mi_fetch_keypage(register MI_INFO *info, MI_KEYDEF *keyinfo,
   {
     DBUG_PRINT("error",("page %lu had wrong page length: %u",
 			(ulong) page, page_size));
+    DBUG_DUMP("page", (char*) tmp, keyinfo->block_length);
     info->last_keypage = HA_OFFSET_ERROR;
     my_errno = HA_ERR_CRASHED;
     tmp = 0;
   }
-  return tmp;
+  DBUG_RETURN(tmp);
 } /* _mi_fetch_keypage */
 
 
 	/* Write a key-page on disk */
 
 int _mi_write_keypage(register MI_INFO *info, register MI_KEYDEF *keyinfo,
-		      my_off_t page, uchar *buff)
+		      my_off_t page, int level, uchar *buff)
 {
   reg3 uint length;
+  DBUG_ENTER("_mi_write_keypage");
+
 #ifndef FAST					/* Safety check */
   if (page < info->s->base.keystart ||
       page+keyinfo->block_length > info->state->key_file_length ||
@@ -71,7 +76,7 @@ int _mi_write_keypage(register MI_INFO *info, register MI_KEYDEF *keyinfo,
 			(long) info->state->key_file_length,
 			(long) page));
     my_errno=EINVAL;
-    return(-1);
+    DBUG_RETURN((-1));
   }
   DBUG_PRINT("page",("write page at: %lu",(long) page,buff));
   DBUG_DUMP("buff",(byte*) buff,mi_getint(buff));
@@ -87,16 +92,18 @@ int _mi_write_keypage(register MI_INFO *info, register MI_KEYDEF *keyinfo,
     length=keyinfo->block_length;
   }
 #endif
-  return (key_cache_write(info->s->kfile,page,(byte*) buff,length,
+  DBUG_RETURN((key_cache_write(info->s->key_cache,
+                         info->s->kfile,page, level, (byte*) buff,length,
 			 (uint) keyinfo->block_length,
 			 (int) ((info->lock_type != F_UNLCK) ||
-				info->s->delay_key_write)));
+				info->s->delay_key_write))));
 } /* mi_write_keypage */
 
 
 	/* Remove page from disk */
 
-int _mi_dispose(register MI_INFO *info, MI_KEYDEF *keyinfo, my_off_t pos)
+int _mi_dispose(register MI_INFO *info, MI_KEYDEF *keyinfo, my_off_t pos,
+                int level)
 {
   my_off_t old_link;
   char buff[8];
@@ -107,7 +114,8 @@ int _mi_dispose(register MI_INFO *info, MI_KEYDEF *keyinfo, my_off_t pos)
   info->s->state.key_del[keyinfo->block_size]=pos;
   mi_sizestore(buff,old_link);
   info->s->state.changed|= STATE_NOT_SORTED_PAGES;
-  DBUG_RETURN(key_cache_write(info->s->kfile,pos,buff,
+  DBUG_RETURN(key_cache_write(info->s->key_cache,
+                              info->s->kfile, pos , level, buff,
 			      sizeof(buff),
 			      (uint) keyinfo->block_length,
 			      (int) (info->lock_type != F_UNLCK)));
@@ -116,7 +124,7 @@ int _mi_dispose(register MI_INFO *info, MI_KEYDEF *keyinfo, my_off_t pos)
 
 	/* Make new page on disk */
 
-my_off_t _mi_new(register MI_INFO *info, MI_KEYDEF *keyinfo)
+my_off_t _mi_new(register MI_INFO *info, MI_KEYDEF *keyinfo, int level)
 {
   my_off_t pos;
   char buff[8];
@@ -135,7 +143,8 @@ my_off_t _mi_new(register MI_INFO *info, MI_KEYDEF *keyinfo)
   }
   else
   {
-    if (!key_cache_read(info->s->kfile,pos,
+    if (!key_cache_read(info->s->key_cache,
+                        info->s->kfile, pos, level,
 			buff,
 			(uint) sizeof(buff),
 			(uint) keyinfo->block_length,0))
