@@ -17,10 +17,7 @@
 /* Update an old row in a MyISAM table */
 
 #include "fulltext.h"
-#ifdef	__WIN__
-#include <errno.h>
-#endif
-
+#include "rt_index.h"
 
 int mi_update(register MI_INFO *info, const byte *oldrec, byte *newrec)
 {
@@ -61,6 +58,7 @@ int mi_update(register MI_INFO *info, const byte *oldrec, byte *newrec)
     goto err_end;			/* Record has changed */
   }
 
+
   /* Calculate and check all unique constraints */
   key_changed=0;
   for (i=0 ; i < share->state.header.uniques ; i++)
@@ -88,7 +86,6 @@ int mi_update(register MI_INFO *info, const byte *oldrec, byte *newrec)
   {
     if (((ulonglong) 1 << i) & share->state.key_map)
     {
-      /* The following code block is for text searching by SerG */
       if (share->keyinfo[i].flag & HA_FULLTEXT )
       {
 	if (_mi_ft_cmp(info,i,oldrec, newrec))
@@ -118,8 +115,8 @@ int mi_update(register MI_INFO *info, const byte *oldrec, byte *newrec)
 	    key_changed|=HA_STATE_WRITTEN;	/* Mark that keyfile changed */
 	  changed|=((ulonglong) 1 << i);
 	  share->keyinfo[i].version++;
-	  if (_mi_ck_delete(info,i,old_key,old_length)) goto err;
-	  if (_mi_ck_write(info,i,new_key,new_length)) goto err;
+	  if (share->keyinfo[i].ck_delete(info,i,old_key,old_length)) goto err;
+	  if (share->keyinfo[i].ck_insert(info,i,new_key,new_length)) goto err;
 	  if (share->base.auto_key == i+1)
 	    auto_key_changed=1;
 	}
@@ -190,7 +187,6 @@ err:
     {
       if (((ulonglong) 1 << i) & changed)
       {
-	/* The following code block is for text searching by SerG */
 	if (share->keyinfo[i].flag & HA_FULLTEXT)
 	{
 	  if ((flag++ && _mi_ft_del(info,i,(char*) new_key,newrec,pos)) ||
