@@ -43,7 +43,7 @@ void pack_dirname(my_string to, const char *from)
   (void) intern_filename(to,from);		/* Change to intern name */
 
 #ifdef FN_DEVCHAR
-  if ((start=strrchr(to,FN_DEVCHAR)) != 0)	/* Skipp device part */
+  if ((start=strrchr(to,FN_DEVCHAR)) != 0)	/* Skip device part */
     start++;
   else
 #endif
@@ -124,6 +124,9 @@ uint cleanup_dirname(register my_string to, const char *from)
   reg4 my_string start;
   char parent[5],				/* for "FN_PARENTDIR" */
        buff[FN_REFLEN+1],*end_parentdir;
+#ifdef BACKSLASH_MBTAIL
+  CHARSET_INFO *fs= fs_character_set();
+#endif
   DBUG_ENTER("cleanup_dirname");
   DBUG_PRINT("enter",("from: '%s'",from));
 
@@ -131,7 +134,7 @@ uint cleanup_dirname(register my_string to, const char *from)
   from_ptr=(my_string) from;
 #ifdef FN_DEVCHAR
   if ((pos=strrchr(from_ptr,FN_DEVCHAR)) != 0)
-  {						/* Skipp device part */
+  {						/* Skip device part */
     length=(uint) (pos-from_ptr)+1;
     start=strnmov(buff,from_ptr,length); from_ptr+=length;
   }
@@ -141,12 +144,21 @@ uint cleanup_dirname(register my_string to, const char *from)
   length=(uint) (strmov(parent+1,FN_PARENTDIR)-parent);
   for (pos=start ; (*pos= *from_ptr++) != 0 ; pos++)
   {
+#ifdef BACKSLASH_MBTAIL
+    uint l;
+    if (use_mb(fs) && (l= my_ismbchar(fs, from_ptr - 1, from_ptr + 2)))
+    {
+      for (l-- ; l ; *++pos= *from_ptr++, l--);
+      start= pos + 1; /* Don't look inside multi-byte char */
+      continue;
+    }
+#endif
     if (*pos == '/')
       *pos = FN_LIBCHAR;
     if (*pos == FN_LIBCHAR)
     {
       if ((uint) (pos-start) > length && bcmp(pos-length,parent,length) == 0)
-      {						/* If .../../; skipp prev */
+      {						/* If .../../; skip prev */
 	pos-=length;
 	if (pos != start)
 	{					 /* not /../ */
@@ -195,7 +207,7 @@ uint cleanup_dirname(register my_string to, const char *from)
 	  pos--;			/* Remove dupplicate '/' */
       }
       else if (pos-start > 1 && pos[-1] == FN_CURLIB && pos[-2] == FN_LIBCHAR)
-	pos-=2;					/* Skipp /./ */
+	pos-=2;					/* Skip /./ */
       else if (pos > buff+1 && pos[-1] == FN_HOMELIB && pos[-2] == FN_LIBCHAR)
       {					/* Found ..../~/  */
 	buff[0]=FN_HOMELIB;
@@ -226,7 +238,7 @@ void symdirget(char *dir)
 {
   char buff[FN_REFLEN];
   char *pos=strend(dir);
-  if (dir[0] && pos[-1] != FN_DEVCHAR && access(dir, F_OK))
+  if (dir[0] && pos[-1] != FN_DEVCHAR && my_access(dir, F_OK))
   {
     File file;
     uint length;
@@ -409,7 +421,7 @@ uint system_filename(my_string to, const char *from)
   libchar_found=0;
   (void) strmov(buff,from);			 /* If to == from */
   from_pos= buff;
-  if ((pos=strrchr(from_pos,FN_DEVCHAR)))	/* Skipp device part */
+  if ((pos=strrchr(from_pos,FN_DEVCHAR)))	/* Skip device part */
   {
     pos++;
     to_pos=strnmov(to,from_pos,(size_s) (pos-from_pos));
@@ -419,7 +431,7 @@ uint system_filename(my_string to, const char *from)
     to_pos=to;
 
   if (from_pos[0] == FN_CURLIB && from_pos[1] == FN_LIBCHAR)
-    from_pos+=2;				/* Skipp './' */
+    from_pos+=2;				/* Skip './' */
   if (strchr(from_pos,FN_LIBCHAR))
   {
     *(to_pos++) = FN_C_BEFORE_DIR;
@@ -487,7 +499,7 @@ my_string intern_filename(my_string to, const char *from)
 
   convert_dirname(buff,from,NullS);		/* change '<>' to '[]' */
   from_pos=buff;
-  if ((pos=strrchr(from_pos,FN_DEVCHAR)))	/* Skipp device part */
+  if ((pos=strrchr(from_pos,FN_DEVCHAR)))	/* Skip device part */
   {
     pos++;
     to_pos=strnmov(to,from_pos,(size_s) (pos-from_pos));
