@@ -23,7 +23,6 @@
 #include <signal.h>
 #include <m_string.h>
 #include <thr_alarm.h>
-#include <assert.h>
 
 #if (defined(__BSD__) || defined(_BSDI_VERSION)) && !defined(HAVE_mit_thread)
 #define SCHED_POLICY SCHED_RR
@@ -98,25 +97,23 @@ void *my_pthread_getspecific_imp(pthread_key_t key)
 #undef pthread_exit
 void my_pthread_exit(void *status)
 {
-  NXThreadId_t tid = NXThreadGetId();
+  NXThreadId_t tid;
   NXContext_t ctx;
-  char name[PATH_MAX] = "";
+  char name[NX_MAX_OBJECT_NAME_LEN+1] = "";
 
-  /* Do not call pthread_exit if it is not a LibC thread */
-  if (tid != 0)
-  {
-    NXThreadGetContext(tid, &ctx);
-    NXContextGetName(ctx, name, PATH_MAX);
+  tid= NXThreadGetId();
+  if (tid == NX_INVALID_THREAD_ID || !tid)
+    return;
+  if (NXThreadGetContext(tid, &ctx) ||
+      NXContextGetName(ctx, name, sizeof(name)-1))
+    return;
 
-    /*
-      "MYSQLD.NLM's LibC Reaper" or "MYSQLD.NLM's main thread"
-      with a debug build of LibC the reaper can have different names
-    */
-    if (!strindex(name, "\'s"))
-    {
-      pthread_exit(status);
-    }
-  }
+  /*
+    "MYSQLD.NLM's LibC Reaper" or "MYSQLD.NLM's main thread"
+    with a debug build of LibC the reaper can have different names
+  */
+  if (!strindex(name, "\'s"))
+    pthread_exit(status);
 }
 #endif
 
@@ -141,10 +138,13 @@ int my_sigwait(const sigset_t *set,int *sig)
 
 /* localtime_r for SCO 3.2V4.2 */
 
-#ifndef HAVE_LOCALTIME_R
+#if !defined(HAVE_LOCALTIME_R) || !defined(HAVE_GMTIME_R)
 
 extern pthread_mutex_t LOCK_localtime_r;
 
+#endif
+
+#if !defined(HAVE_LOCALTIME_R)
 struct tm *localtime_r(const time_t *clock, struct tm *res)
 {
   struct tm *tmp;
@@ -156,6 +156,22 @@ struct tm *localtime_r(const time_t *clock, struct tm *res)
 }
 #endif
 
+#if !defined(HAVE_GMTIME_R)
+/* 
+  Reentrant version of standard gmtime() function. 
+  Needed on some systems which don't implement it.
+*/
+
+struct tm *gmtime_r(const time_t *clock, struct tm *res)
+{
+  struct tm *tmp;
+  pthread_mutex_lock(&LOCK_localtime_r);
+  tmp= gmtime(clock);
+  *res= *tmp;
+  pthread_mutex_unlock(&LOCK_localtime_r);
+  return res;
+}
+#endif
 
 /****************************************************************************
 ** Replacement of sigwait if the system doesn't have one (like BSDI 3.0)
