@@ -18,6 +18,7 @@ SILENT=0
 MACHINE=
 TMP=/tmp
 SUFFIX=""
+NDBCLUSTER=
 
 parse_arguments() {
   for arg do
@@ -28,6 +29,7 @@ parse_arguments() {
       --no-strip) STRIP=0 ;;
       --machine=*)  MACHINE=`echo "$arg" | sed -e "s;--machine=;;"` ;;
       --silent)   SILENT=1 ;;
+      --with-ndbcluster) NDBCLUSTER=1 ;;
       *)
 	echo "Unknown argument '$arg'"
 	exit 1
@@ -66,7 +68,7 @@ esac
 mkdir $BASE $BASE/bin $BASE/docs \
  $BASE/include $BASE/lib $BASE/support-files $BASE/share $BASE/scripts \
  $BASE/mysql-test $BASE/mysql-test/t  $BASE/mysql-test/r \
- $BASE/mysql-test/include $BASE/mysql-test/std_data
+ $BASE/mysql-test/include $BASE/mysql-test/std_data $BASE/mysql-test/lib
 
 if [ $BASE_SYSTEM != "netware" ] ; then
  mkdir $BASE/share/mysql $BASE/tests $BASE/sql-bench $BASE/man \
@@ -100,12 +102,15 @@ BIN_FILES="extra/comp_err$BS extra/replace$BS extra/perror$BS \
   isam/isamchk$BS isam/pack_isam$BS \
   myisam/myisamchk$BS myisam/myisampack$BS myisam/myisamlog$BS \
   myisam/myisam_ftdump$BS \
-  sql/mysqld$BS \
+  sql/mysqld$BS sql/mysql_tzinfo_to_sql$BS \
   client/mysql$BS client/mysqlshow$BS client/mysqladmin$BS \
   client/mysqldump$BS client/mysqlimport$BS \
   client/mysqltest$BS client/mysqlcheck$BS \
-  client/mysqlbinlog$BS 
-";
+  client/mysqlbinlog$BS \
+  tests/mysql_client_test$BS \
+  libmysqld/examples/mysql_client_test_embedded$BS \
+  libmysqld/examples/mysqltest_embedded$BS \
+  ";
 
 # Platform-specific bin dir files:
 if [ $BASE_SYSTEM = "netware" ] ; then
@@ -124,6 +129,9 @@ else
     client/.libs/mysqltest client/.libs/mysqlcheck \
     client/.libs/mysqlbinlog client/.libs/mysqlmanagerc \
     client/.libs/mysqlmanager-pwgen tools/.libs/mysqlmanager \
+    tests/.libs/mysql_client_test \
+    libmysqld/examples/.libs/mysql_client_test_embedded \
+    libmysqld/examples/.libs/mysqltest_embedded \
   ";
 fi
 
@@ -192,20 +200,13 @@ if [ $BASE_SYSTEM != "netware" ] ; then
 fi
 
 $CP support-files/* $BASE/support-files
-
-if [ $BASE_SYSTEM = "netware" ] ; then
-  rm -f $BASE/support-files/magic \
-        $BASE/support-files/mysql.server \
-        $BASE/support-files/mysql*.spec \
-        $BASE/support-files/mysql-log-rotate \
-        $BASE/support-files/binary-configure
-fi
+$CP scripts/*.sql $BASE/share
 
 $CP -r sql/share/* $MYSQL_SHARE
 rm -f $MYSQL_SHARE/Makefile* $MYSQL_SHARE/*/*.OLD
 
 for i in mysql-test/mysql-test-run mysql-test/install_test_db \
-         mysql-test/README \
+         mysql-test/mysql-test-run.pl mysql-test/README \
          netware/mysql_test_run.nlm netware/install_test_db.ncf
 do
   if [ -f $i ]
@@ -214,8 +215,10 @@ do
    fi
 done
 
+$CP mysql-test/lib/*.pl  $BASE/mysql-test/lib
+$CP mysql-test/lib/*.sql $BASE/mysql-test/lib
 $CP mysql-test/include/*.inc $BASE/mysql-test/include
-$CP mysql-test/std_data/*.dat mysql-test/std_data/*.001 $BASE/mysql-test/std_data
+$CP mysql-test/std_data/*.dat mysql-test/std_data/*.*001 $BASE/mysql-test/std_data
 $CP mysql-test/std_data/des_key_file $BASE/mysql-test/std_data
 $CP mysql-test/t/*test mysql-test/t/*.opt mysql-test/t/*.slave-mi mysql-test/t/*.sh $BASE/mysql-test/t
 $CP mysql-test/r/*result mysql-test/r/*.require $BASE/mysql-test/r
@@ -223,23 +226,42 @@ $CP mysql-test/r/*result mysql-test/r/*.require $BASE/mysql-test/r
 if [ $BASE_SYSTEM != "netware" ] ; then
   chmod a+x $BASE/bin/*
   $CP scripts/* $BASE/bin
-  $BASE/bin/replace \@localstatedir\@ ./data \@bindir\@ ./bin \@scriptdir\@ ./bin \@libexecdir\@ ./bin \@sbindir\@ ./bin \@prefix\@ . \@HOSTNAME\@ @HOSTNAME@ < $SOURCE/scripts/mysql_install_db.sh > $BASE/scripts/mysql_install_db
+  $BASE/bin/replace \@localstatedir\@ ./data \@bindir\@ ./bin \@scriptdir\@ ./bin \@libexecdir\@ ./bin \@sbindir\@ ./bin \@prefix\@ . \@HOSTNAME\@ @HOSTNAME@ \@pkgdatadir\@ ./support-files < $SOURCE/scripts/mysql_install_db.sh > $BASE/scripts/mysql_install_db
   $BASE/bin/replace \@prefix\@ /usr/local/mysql \@bindir\@ ./bin \@MYSQLD_USER\@ root \@localstatedir\@ /usr/local/mysql/data \@HOSTNAME\@ @HOSTNAME@ < $SOURCE/support-files/mysql.server.sh > $BASE/support-files/mysql.server
   $BASE/bin/replace /my/gnu/bin/hostname /bin/hostname -- $BASE/bin/mysqld_safe
   mv $BASE/support-files/binary-configure $BASE/configure
   chmod a+x $BASE/bin/* $BASE/scripts/* $BASE/support-files/mysql-* $BASE/support-files/mysql.server $BASE/configure
   $CP -r sql-bench/* $BASE/sql-bench
   rm -f $BASE/sql-bench/*.sh $BASE/sql-bench/Makefile* $BASE/lib/*.la
+  rm -f $BASE/bin/*.sql
 fi
 
 rm -f $BASE/bin/Makefile* $BASE/bin/*.in $BASE/bin/*.sh $BASE/bin/mysql_install_db $BASE/bin/make_binary_distribution $BASE/bin/setsomevars $BASE/support-files/Makefile* $BASE/support-files/*.sh
 
 
+#
+# Copy system dependent files
+#
+if [ $BASE_SYSTEM = "netware" ] ; then
+  echo "CREATE DATABASE mysql;" > $BASE/bin/init_db.sql
+  echo "CREATE DATABASE test;" >> $BASE/bin/init_db.sql
+  sh ./scripts/mysql_create_system_tables.sh real "" "%" 0 >> $BASE/bin/init_db.sql
+  sh ./scripts/mysql_create_system_tables.sh test "" "%" 0 > $BASE/bin/test_db.sql
+#  cp ./netware/static_init_db.sql ./netware/init_db.sql
+#  ./scripts/fill_help_tables < ./Docs/manual.texi >> ./netware/init_db.sql
+fi
+
 #
 # Remove system dependent files
 #
 if [ $BASE_SYSTEM = "netware" ] ; then
-    rm -f $BASE/MySQLEULA.txt
+  rm -f $BASE/support-files/magic \
+        $BASE/support-files/mysql.server \
+        $BASE/support-files/mysql*.spec \
+        $BASE/support-files/mysql-log-rotate \
+        $BASE/support-files/binary-configure \
+        $BASE/INSTALL-BINARY \
+        $BASE/MySQLEULA.txt
 else
     rm -f $BASE/README.NW
 fi
@@ -254,6 +276,18 @@ if [ -d $BASE/sql-bench/SCCS ] ; then
   find $BASE/sql-bench -name SCCS -print | xargs rm -r -f
 fi
 
+# NDB Cluster
+if [ x$NDBCLUSTER = x1 ]; then
+  ( cd ndb            ; @MAKE@ DESTDIR=$BASE/ndb-stage install )
+  ( cd mysql-test/ndb ; @MAKE@ DESTDIR=$BASE/ndb-stage install )
+  $CP $BASE/ndb-stage@bindir@/* $BASE/bin/.
+  $CP $BASE/ndb-stage@libexecdir@/* $BASE/bin/.
+  $CP $BASE/ndb-stage@pkglibdir@/* $BASE/lib/.
+  $CP -r $BASE/ndb-stage@pkgincludedir@/ndb $BASE/include
+  $CP -r $BASE/ndb-stage@prefix@/mysql-test/ndb $BASE/mysql-test/. || exit 1
+  rm -rf $BASE/ndb-stage
+fi
+
 # Use the override --machine if present
 if [ -n "$MACHINE" ] ; then
   machine=$MACHINE
@@ -279,7 +313,7 @@ then
   gcclib=`@CC@ --print-libgcc-file`
   if test $? -ne 0
   then
-    echo "Warning: Couldn't find libgcc.a!"
+    print "Warning: Couldn't find libgcc.a!"
   else
     $CP $gcclib libmygcc.a
   fi
@@ -339,9 +373,6 @@ if [ $BASE_SYSTEM != "netware" ] ; then
   echo "Compressing archive"
   rm -f $NEW_NAME.tar.gz
   gzip -9 $NEW_NAME.tar
-  echo "Removing temporary directory"
-  rm -r -f $BASE
-  
   echo "$NEW_NAME.tar.gz created"
 else
 
@@ -352,9 +383,8 @@ else
   cd $TMP
   if test -e "$SOURCE/$NEW_NAME.zip"; then rm $SOURCE/$NEW_NAME.zip; fi
   zip -r $SOURCE/$NEW_NAME.zip $NEW_NAME
-  echo "Removing temporary directory"
-  rm -r -f $BASE
-
   echo "$NEW_NAME.zip created"
 
 fi
+echo "Removing temporary directory"
+rm -r -f $BASE
