@@ -60,11 +60,19 @@ fix_path ()
 
 get_full_path ()
 {
-  case $1 in
-    /*)	echo "$1";;
-    ./*) tmp=`pwd`/$1; echo $tmp | sed -e 's;/\./;/;' ;;
-     *) which $1 ;;
-   esac
+  file=$1
+
+  # if the file is a symlink, try to resolve it
+  if [ -h $file ];
+  then
+    file=`ls -l $file | awk '{ print $NF }'`
+  fi
+
+  case $file in
+    /*) echo "$file";;
+    */*) tmp=`pwd`/$file; echo $tmp | sed -e 's;/\./;/;' ;;
+    *) which $file ;;
+  esac
 }
 
 me=`get_full_path $0`
@@ -82,17 +90,18 @@ version='@VERSION@'
 socket='@MYSQL_UNIX_ADDR@'
 port='@MYSQL_TCP_PORT@'
 ldflags='@LDFLAGS@'
-client_libs='@CLIENT_LIBS@'
 
 # Create options
 
-libs="$ldflags -L$pkglibdir -lmysqlclient $client_libs"
+libs="$ldflags -L$pkglibdir -lmysqlclient @ZLIB_DEPS@ @NON_THREADED_LIBS@"
+libs="$libs @openssl_libs@ @STATIC_NSS_FLAGS@"
 libs=`echo "$libs" | sed -e 's;  \+; ;g' | sed -e 's;^ *;;' | sed -e 's; *\$;;'`
-libs_r="$ldflags -L$pkglibdir -lmysqlclient_r @LIBS@ @openssl_libs@"
+
+libs_r="$ldflags -L$pkglibdir -lmysqlclient_r @ZLIB_DEPS@ @LIBS@ @openssl_libs@"
 libs_r=`echo "$libs_r" | sed -e 's;  \+; ;g' | sed -e 's;^ *;;' | sed -e 's; *\$;;'`
 cflags="-I$pkgincludedir @CFLAGS@ " #note: end space!
 include="-I$pkgincludedir"
-embedded_libs="$ldflags -L$pkglibdir -lmysqld @LIBS@ @WRAPLIBS@ @innodb_system_libs@"
+embedded_libs="$ldflags -L$pkglibdir -lmysqld @LIBS@ @WRAPLIBS@ @innodb_system_libs@ $client_libs"
 embedded_libs=`echo "$embedded_libs" | sed -e 's;  \+; ;g' | sed -e 's;^ *;;' | sed -e 's; *\$;;'`
 
 # Remove some options that a client doesn't have to care about
@@ -100,7 +109,9 @@ for remove in DDBUG_OFF DSAFEMALLOC USAFEMALLOC DSAFE_MUTEX \
               DPEDANTIC_SAFEMALLOC DUNIV_MUST_NOT_INLINE DFORCE_INIT_OF_VARS \
               DEXTRA_DEBUG DHAVE_purify 'O[0-9]' 'W[-A-Za-z]*'
 do
-  cflags=`echo "$cflags"|sed -e "s/-$remove  *//g"` 
+  # The first option we might strip will always have a space before it because
+  # we set -I$pkgincludedir as the first option
+  cflags=`echo "$cflags"|sed -e "s/ -$remove  */ /g"` 
 done
 cflags=`echo "$cflags"|sed -e 's/ *\$//'` 
 
