@@ -21,8 +21,6 @@
 
 struct st_des_keyschedule des_keyschedule[10];
 uint   des_default_key;
-pthread_mutex_t LOCK_des_key_file;
-static int initialized;
 
 /*
  Function which loads DES keys from plaintext file into memory on MySQL
@@ -34,6 +32,8 @@ static int initialized;
 	1	Error   
 */
 
+#define des_cs	&my_charset_latin1
+
 bool
 load_des_key_file(const char *file_name)
 {
@@ -43,12 +43,6 @@ load_des_key_file(const char *file_name)
   DBUG_ENTER("load_des_key_file");
   DBUG_PRINT("enter",("name: %s",file_name));
 
-  if (!initialized)
-  {
-    initialized=1;
-    pthread_mutex_init(&LOCK_des_key_file,MY_MUTEX_INIT_FAST);
-  }
-
   VOID(pthread_mutex_lock(&LOCK_des_key_file));
   if ((file=my_open(file_name,O_RDONLY | O_BINARY ,MYF(MY_WME))) < 0 ||
       init_io_cache(&io, file, IO_SIZE*2, READ_CACHE, 0, 0, MYF(MY_WME)))
@@ -70,9 +64,10 @@ load_des_key_file(const char *file_name)
     {
       offset=(char) (offset - '0');
       // Remove newline and possible other control characters
-      for (start=buf+1 ; isspace(*start) ; start++) ;
+      for (start=buf+1 ; my_isspace(des_cs, *start) ; start++) ;
       end=buf+length;
-      for  (end=strend(buf) ; end > start && !isgraph(end[-1]) ; end--) ;
+      for  (end=strend(buf) ; 
+            end > start && !my_isgraph(des_cs, end[-1]) ; end--) ;
 
       if (start != end)
       {
@@ -104,15 +99,4 @@ error:
   VOID(pthread_mutex_unlock(&LOCK_des_key_file));
   DBUG_RETURN(result);
 }
-
-
-void free_des_key_file()
-{
-  if (initialized)
-  {
-    initialized= 01;
-    pthread_mutex_destroy(&LOCK_des_key_file);
-  }
-}
-
 #endif /* HAVE_OPENSSL */
