@@ -27,9 +27,6 @@
 extern "C" {					// Because of SCO 3.2V4.2
 #endif
 #if !defined( __WIN__) && !defined(OS2)
-#if !defined(__NETWARE__)
-#include <sys/resource.h>
-#endif /* __NETWARE__ */
 #ifdef HAVE_SYS_UN_H
 #include <sys/un.h>
 #endif
@@ -61,22 +58,27 @@ bool hostname_cache_init()
 {
   host_entry tmp;
   uint offset= (uint) ((char*) (&tmp.ip) - (char*) &tmp);
-  (void) pthread_mutex_init(&LOCK_hostname,MY_MUTEX_INIT_SLOW);
-
   if (!(hostname_cache=new hash_filo(HOST_CACHE_SIZE, offset,
 				     sizeof(struct in_addr),NULL,
-				     (hash_free_key) free)))
+				     (hash_free_key) free,
+				     &my_charset_latin1)))
     return 1;
   hostname_cache->clear();
+  (void) pthread_mutex_init(&LOCK_hostname,MY_MUTEX_INIT_SLOW);
   return 0;
 }
 
 void hostname_cache_free()
 {
-  (void) pthread_mutex_destroy(&LOCK_hostname);
-  delete hostname_cache;
+  if (hostname_cache)
+  {
+    (void) pthread_mutex_destroy(&LOCK_hostname);
+    delete hostname_cache;
+    hostname_cache= 0;
+  }
 }
 
+
 static void add_hostname(struct in_addr *in,const char *name)
 {
   if (!(specialflag & SPECIAL_NO_HOST_CACHE))
@@ -175,7 +177,14 @@ my_string ip_to_hostname(struct in_addr *in, uint *errors)
 				 &tmp_errno)))
   {
     DBUG_PRINT("error",("gethostbyname_r returned %d",tmp_errno));
-    add_wrong_ip(in);
+    /*
+      Don't cache responses when the DSN server is down, as otherwise
+      transient DNS failure may leave any number of clients (those
+      that attempted to connect during the outage) unable to connect
+      indefinitely.
+    */
+    if (tmp_errno == HOST_NOT_FOUND || tmp_errno == NO_DATA)
+      add_wrong_ip(in);
     my_gethostbyname_r_free();
     DBUG_RETURN(0);
   }
@@ -198,13 +207,17 @@ my_string ip_to_hostname(struct in_addr *in, uint *errors)
   {
     VOID(pthread_mutex_unlock(&LOCK_hostname));
     DBUG_PRINT("error",("gethostbyaddr returned %d",errno));
-    goto err;
+
+    if (errno == HOST_NOT_FOUND || errno == NO_DATA)
+      goto add_wrong_ip_and_return;
+    /* Failure, don't cache responce */
+    DBUG_RETURN(0);
   }
   if (!hp->h_name[0])				// Don't allow empty hostnames
   {
     VOID(pthread_mutex_unlock(&LOCK_hostname));
     DBUG_PRINT("error",("Got an empty hostname"));
-    goto err;
+    goto add_wrong_ip_and_return;
   }
   if (!(name=my_strdup(hp->h_name,MYF(0))))
   {
@@ -223,15 +236,15 @@ my_string ip_to_hostname(struct in_addr *in, uint *errors)
 
   /* Don't accept hostnames that starts with digits because they may be
      false ip:s */
-  if (isdigit(name[0]))
+  if (my_isdigit(&my_charset_latin1,name[0]))
   {
     char *pos;
-    for (pos= name+1 ; isdigit(*pos); pos++) ;
+    for (pos= name+1 ; my_isdigit(&my_charset_latin1,*pos); pos++) ;
     if (*pos == '.')
     {
       DBUG_PRINT("error",("mysqld doesn't accept hostnames that starts with a number followed by a '.'"));
       my_free(name,MYF(0));
-      goto err;
+      goto add_wrong_ip_and_return;
     }
   }
 
@@ -247,7 +260,7 @@ my_string ip_to_hostname(struct in_addr *in, uint *errors)
   DBUG_PRINT("error",("Couldn't verify hostname with gethostbyname"));
   my_free(name,MYF(0));
 
-err:
+add_wrong_ip_and_return:
   add_wrong_ip(in);
   DBUG_RETURN(0);
 }
