@@ -27,7 +27,7 @@
 #include "ha_innodb.h"
 #include "sql_select.h"
 
-int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, ORDER *order,
+int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, SQL_LIST *order,
                  ha_rows limit, ulong options)
 {
   int		error;
@@ -35,27 +35,33 @@ int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, ORDER *order,
   SQL_SELECT	*select=0;
   READ_RECORD	info;
   bool 		using_limit=limit != HA_POS_ERROR;
-  bool		transactional_table, log_delayed, safe_update;
+  bool		transactional_table, log_delayed, safe_update, const_cond; 
   ha_rows	deleted;
   DBUG_ENTER("mysql_delete");
 
-  if (((safe_update=thd->options & OPTION_SAFE_UPDATES)) && !conds)
-  {
-    send_error(&thd->net,ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE);
-    DBUG_RETURN(1);
-  }
-
-  if (!(table = open_ltable(thd, table_list, table_list->lock_type)))
+  if ((open_and_lock_tables(thd, table_list)))
     DBUG_RETURN(-1);
+  table= table_list->table;
   table->file->info(HA_STATUS_VARIABLE | HA_STATUS_NO_LOCK);
   thd->proc_info="init";
   table->map=1;
-  if (setup_conds(thd,table_list,&conds) || setup_ftfuncs(thd))
-    DBUG_RETURN(-1);
+
+  if ((error= mysql_prepare_delete(thd, table_list, &conds)))
+    DBUG_RETURN(error);
+
+  const_cond= (!conds || conds->const_item());
+  safe_update=test(thd->options & OPTION_SAFE_UPDATES);
+  if (safe_update && const_cond)
+  {
+    send_error(thd,ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE);
+    DBUG_RETURN(1);
+  }
+
+  thd->lex->select_lex.no_error= thd->lex->ignore;
 
   /* Test if the user wants to delete all rows */
-  if (!using_limit && (!conds || (conds->const_item() && conds->val_int())) &&
-      !(specialflag & (SPECIAL_NO_NEW_FUNC | SPECIAL_SAFE_MODE)) && !safe_update)
+  if (!using_limit && const_cond && (!conds || conds->val_int()) &&
+      !(specialflag & (SPECIAL_NO_NEW_FUNC | SPECIAL_SAFE_MODE)))
   {
     deleted= table->file->records;
     if (!(error=table->file->delete_all_rows()))
@@ -72,35 +78,35 @@ int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, ORDER *order,
     /* Handler didn't support fast delete; Delete rows one by one */
   }
 
-  table->used_keys=table->quick_keys=0;		// Can't use 'only index'
+  table->used_keys.clear_all();
+  table->quick_keys.clear_all();		// Can't use 'only index'
   select=make_select(table,0,0,conds,&error);
   if (error)
     DBUG_RETURN(-1);
-  if ((select && select->check_quick(thd,
-				     test(thd->options & OPTION_SAFE_UPDATES),
-				     limit)) ||
-      !limit)
+  if ((select && select->check_quick(thd, safe_update, limit)) || !limit)
   {
     delete select;
-    send_ok(&thd->net,0L);
+    free_underlaid_joins(thd, &thd->lex->select_lex);
+    send_ok(thd,0L);
     DBUG_RETURN(0);				// Nothing to delete
   }
 
   /* If running in safe sql mode, don't allow updates without keys */
-  if (!table->quick_keys)
+  if (table->quick_keys.is_clear_all())
   {
-    thd->lex.select_lex.options|=QUERY_NO_INDEX_USED;
+    thd->server_status|=SERVER_QUERY_NO_INDEX_USED;
     if (safe_update && !using_limit)
     {
       delete select;
-      send_error(&thd->net,ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE);
+      free_underlaid_joins(thd, &thd->lex->select_lex);
+      send_error(thd,ER_UPDATE_WITHOUT_KEY_IN_SAFE_MODE);
       DBUG_RETURN(1);
     }
   }
   if (options & OPTION_QUICK)
     (void) table->file->extra(HA_EXTRA_QUICK);
 
-  if (order)
+  if (order && order->elements)
   {
     uint         length;
     SORT_FIELD  *sortorder;
@@ -113,16 +119,19 @@ int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, ORDER *order,
     tables.table = table;
     tables.alias = table_list->alias;
 
-    table->io_cache = (IO_CACHE *) my_malloc(sizeof(IO_CACHE),
+    table->sort.io_cache = (IO_CACHE *) my_malloc(sizeof(IO_CACHE),
                                              MYF(MY_FAE | MY_ZEROFILL));
-    if (setup_order(thd, &tables, fields, all_fields, order) ||
-        !(sortorder=make_unireg_sortorder(order, &length)) ||
-        (table->found_records = filesort(table, sortorder, length,
-					 select, 0L, HA_POS_ERROR,
-					 &examined_rows))
-        == HA_POS_ERROR)
+      if (thd->lex->select_lex.setup_ref_array(thd, order->elements) ||
+	  setup_order(thd, thd->lex->select_lex.ref_pointer_array, &tables,
+		      fields, all_fields, (ORDER*) order->first) ||
+	  !(sortorder=make_unireg_sortorder((ORDER*) order->first, &length)) ||
+	  (table->sort.found_records = filesort(thd, table, sortorder, length,
+					   select, HA_POS_ERROR,
+					   &examined_rows))
+	  == HA_POS_ERROR)
     {
       delete select;
+      free_underlaid_joins(thd, &thd->lex->select_lex);
       DBUG_RETURN(-1);			// This will force out message
     }
     /*
@@ -135,11 +144,13 @@ int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, ORDER *order,
 
   init_read_record(&info,thd,table,select,1,1);
   deleted=0L;
-  init_ftfuncs(thd,1);
+  init_ftfuncs(thd, &thd->lex->select_lex, 1);
   thd->proc_info="updating";
-  while (!(error=info.read_record(&info)) && !thd->killed)
+  while (!(error=info.read_record(&info)) && !thd->killed &&
+	 !thd->net.report_error)
   {
-    if (!(select && select->skipp_record()))
+    // thd->net.report_error is tested to disallow delete row on error
+    if (!(select && select->skip_record())&& !thd->net.report_error )
     {
       if (!(error=table->file->delete_row(table->record[0])))
       {
@@ -177,17 +188,35 @@ int mysql_delete(THD *thd, TABLE_LIST *table_list, COND *conds, ORDER *order,
     (void) table->file->extra(HA_EXTRA_NORMAL);
 
 cleanup:
+  /*
+    Invalidate the table in the query cache if something changed. This must
+    be before binlog writing and ha_autocommit_...
+  */
+  if (deleted)
+  {
+    query_cache_invalidate3(thd, table_list, 1);
+  }
+
+  delete select;
   transactional_table= table->file->has_transactions();
   log_delayed= (transactional_table || table->tmp_table);
-  if (deleted && (error <= 0 || !transactional_table))
+  /*
+    We write to the binary log even if we deleted no row, because maybe the
+    user is using this command to ensure that a table is clean on master *and
+    on slave*. Think of the case of a user having played separately with the
+    master's table and slave's table and wanting to take a fresh identical
+    start now.
+    error < 0 means "really no error". error <= 0 means "maybe some error".
+  */
+  if ((deleted || (error < 0)) && (error <= 0 || !transactional_table))
   {
     mysql_update_log.write(thd,thd->query, thd->query_length);
     if (mysql_bin_log.is_open())
     {
       if (error <= 0)
         thd->clear_error();
-      Query_log_event qinfo(thd, thd->query, thd->query_length, 
-			    log_delayed);
+      Query_log_event qinfo(thd, thd->query, thd->query_length,
+			    log_delayed, FALSE);
       if (mysql_bin_log.write(&qinfo) && transactional_table)
 	error=1;
     }
@@ -200,26 +229,51 @@ cleanup:
       error=1;
   }
 
-  /*
-    Store table for future invalidation  or invalidate it in
-    the query cache if something changed
-  */
-  if (deleted)
-  {
-    query_cache_invalidate3(thd, table_list, 1);
-  }
   if (thd->lock)
   {
     mysql_unlock_tables(thd, thd->lock);
     thd->lock=0;
   }
-  delete select;
-  if (error >= 0)				// Fatal error
-    send_error(&thd->net,thd->killed ? ER_SERVER_SHUTDOWN: 0);
+  free_underlaid_joins(thd, &thd->lex->select_lex);
+  if (error >= 0 || thd->net.report_error)
+    send_error(thd,thd->killed ? ER_SERVER_SHUTDOWN: 0);
   else
   {
-    send_ok(&thd->net,deleted);
+    send_ok(thd,deleted);
     DBUG_PRINT("info",("%d records deleted",deleted));
+  }
+       DBUG_RETURN(0);
+}
+
+
+/*
+  Prepare items in DELETE statement
+
+  SYNOPSIS
+    mysql_prepare_delete()
+    thd			- thread handler
+    table_list		- global table list
+    conds		- conditions
+
+  RETURN VALUE
+    0  - OK
+    1  - error (message is sent to user)
+    -1 - error (message is not sent to user)
+*/
+int mysql_prepare_delete(THD *thd, TABLE_LIST *table_list, Item **conds)
+{
+  TABLE_LIST *delete_table_list= ((TABLE_LIST*) thd->lex->
+				  select_lex.table_list.first);
+  DBUG_ENTER("mysql_prepare_delete");
+
+  if (setup_conds(thd, delete_table_list, conds) || 
+      setup_ftfuncs(&thd->lex->select_lex))
+    DBUG_RETURN(-1);
+  if (find_real_table_in_list(table_list->next, 
+			      table_list->db, table_list->real_name))
+  {
+    my_error(ER_UPDATE_TABLE_USED, MYF(0), table_list->real_name);
+    DBUG_RETURN(-1);
   }
   DBUG_RETURN(0);
 }
@@ -248,9 +302,10 @@ multi_delete::multi_delete(THD *thd_arg, TABLE_LIST *dt,
 
 
 int
-multi_delete::prepare(List<Item> &values)
+multi_delete::prepare(List<Item> &values, SELECT_LEX_UNIT *u)
 {
   DBUG_ENTER("multi_delete::prepare");
+  unit= u;
   do_delete= 1;
   thd->proc_info="deleting from main table";
   DBUG_RETURN(0);
@@ -285,7 +340,7 @@ multi_delete::initialize_tables(JOIN *join)
       tbl->no_keyread=1;
       /* Don't use record cache */
       tbl->no_cache= 1;
-      tbl->used_keys= 0;
+      tbl->used_keys.clear_all();
       if (tbl->file->has_transactions())
 	log_delayed= transactional_tables= 1;
       else if (tbl->tmp_table != NO_TMP_TABLE)
@@ -304,8 +359,8 @@ multi_delete::initialize_tables(JOIN *join)
 				  table->file->ref_length,
 				  MEM_STRIP_BUF_SIZE);
   }
-  init_ftfuncs(thd,1);
-  DBUG_RETURN(thd->fatal_error != 0);
+  init_ftfuncs(thd, thd->lex->current_select, 1);
+  DBUG_RETURN(thd->is_fatal_error != 0);
 }
 
 
@@ -377,7 +432,7 @@ void multi_delete::send_error(uint errcode,const char *err)
   DBUG_ENTER("multi_delete::send_error");
 
   /* First send error what ever it is ... */
-  ::send_error(&thd->net,errcode,err);
+  ::send_error(thd,errcode,err);
 
   /* If nothing deleted return */
   if (!deleted)
@@ -487,11 +542,20 @@ bool multi_delete::send_eof()
   /* reset used flags */
   thd->proc_info="end";
 
+  /*
+    We must invalidate the query cache before binlog writing and
+    ha_autocommit_...
+  */
+  if (deleted)
+    query_cache_invalidate3(thd, delete_tables, 1);
+
   /*
     Write the SQL statement to the binlog if we deleted
     rows and we succeeded, or also in an error case when there
     was a non-transaction-safe table involved, since
     modifications in it cannot be rolled back.
+    Note that if we deleted nothing we don't write to the binlog (TODO:
+    fix this).
   */
   if (deleted && (error <= 0 || normal_tables))
   {
@@ -501,7 +565,7 @@ bool multi_delete::send_eof()
       if (error <= 0)
         thd->clear_error();
       Query_log_event qinfo(thd, thd->query, thd->query_length,
-			    log_delayed);
+			    log_delayed, FALSE);
       if (mysql_bin_log.write(&qinfo) && !normal_tables)
 	local_error=1;  // Log write failed: roll back the SQL statement
     }
@@ -512,14 +576,11 @@ bool multi_delete::send_eof()
   if (transactional_tables)
     if (ha_autocommit_or_rollback(thd,local_error > 0))
       local_error=1;
-  
-  if (deleted)
-    query_cache_invalidate3(thd, delete_tables, 1);
 
   if (local_error)
-    ::send_error(&thd->net);
+    ::send_error(thd);
   else
-    ::send_ok(&thd->net,deleted);
+    ::send_ok(thd, deleted);
   return 0;
 }
 
@@ -556,7 +617,6 @@ int mysql_truncate(THD *thd, TABLE_LIST *table_list, bool dont_send_ok)
     TABLE *table= *table_ptr;
     table->file->info(HA_STATUS_AUTO | HA_STATUS_NO_LOCK);
     db_type table_type=table->db_type;
-
     strmov(path,table->path);
     *table_ptr= table->next;			// Unlink table from list
     close_temporary(table,0);
@@ -567,8 +627,8 @@ int mysql_truncate(THD *thd, TABLE_LIST *table_list, bool dont_send_ok)
 					     table_list->real_name, 1))))
       (void) rm_temporary_table(table_type, path);
     /*
-      Sasha: if we return here we will not have binloged the truncation and
-      we will not send_ok() to the client.
+      If we return here we will not have logged the truncation to the bin log
+      and we will not send_ok() to the client.
     */
     goto end; 
   }
@@ -589,13 +649,15 @@ int mysql_truncate(THD *thd, TABLE_LIST *table_list, bool dont_send_ok)
     if (!ha_supports_generate(table_type))
     {
       /* Probably InnoDB table */
-      table_list->lock_type= TL_WRITE;
       ulong save_options= thd->options;
+      table_list->lock_type= TL_WRITE;
       thd->options&= ~(ulong) (OPTION_BEGIN | OPTION_NOT_AUTOCOMMIT);
-      int res= mysql_delete(thd, table_list, (COND*) 0, (ORDER*) 0,
-			       HA_POS_ERROR, 0);
+      ha_enable_transaction(thd, FALSE);
+      error= mysql_delete(thd, table_list, (COND*) 0, (SQL_LIST*) 0,
+			  HA_POS_ERROR, 0);
+      ha_enable_transaction(thd, TRUE);
       thd->options= save_options;
-      DBUG_RETURN(res);
+      DBUG_RETURN(error);
     }
     if (lock_and_wait_for_table_name(thd, table_list))
       DBUG_RETURN(-1);
@@ -615,10 +677,10 @@ end:
       {
         thd->clear_error();
 	Query_log_event qinfo(thd, thd->query, thd->query_length,
-			      thd->tmp_table);
+			      thd->tmp_table, FALSE);
 	mysql_bin_log.write(&qinfo);
       }
-      send_ok(&thd->net);		// This should return record count
+      send_ok(thd);		// This should return record count
     }
     VOID(pthread_mutex_lock(&LOCK_open));
     unlock_table_name(thd, table_list);
